from PIL import Image
import os
import json
import sys

def imConv(imPath):
    res = []
    im = Image.open(imPath)
    width, height = im.size
    for x in range(width):
        for y in range(height):
            color = im.getpixel([x, y])
            if color[:-1] == (0,0,0):
                res.append([x,y])
    vlines = res.copy()
    vlines.sort(key=lambda x: x[0])
    fvlines = []
    # TODO: Change to pixel/point
    for line in vlines:
        if len(fvlines) > 0:
            if ((fvlines[-1][-1][1] + 1) == line[1]):
                fvlines[-1].append(line)
            else:
                fvlines.append([line])
        else:
            fvlines.append([line])
    vsegments = []
    for line in fvlines:
        append = -1
        for it, segment in enumerate(vsegments):
            if line[0][0] == segment[0]:
                append = it
        if append == -1:
            vsegments.append([line[0][0], len(line)])
        else:
            vsegments[it].append(len(line))            
    hlines = res.copy()
    hlines.sort(key=lambda x: x[1])
    fhlines = []
    for line in hlines:
        if len(fhlines) > 0:
            if ((fhlines[-1][-1][0] + 1) == line[0]):
                fhlines[-1].append(line)
            else:
                fhlines.append([line])
        else:
            fhlines.append([line])
    hsegments = []
    for line in fhlines:
        append = -1
        for it, segment in enumerate(hsegments):
            if line[0][1] == segment[0]:
                append = it
        if append == -1:
            hsegments.append([line[0][1], len(line)])
        else:
            hsegments[it].append(len(line))            
            
    ob = {
        "points": res,
        "horizontals": hsegments,
        "verticals": vsegments,
        "size": [width, height]
    }
    return ob
    
def json2file(data, path):
    with open(path, "w") as f:
        f.write(json.dumps(data))

def loadImages(path="nonograms"):
    mainImages = []
    for folder in sorted(os.listdir(path)):
        spath = path + "/" + folder
        if os.path.isdir(spath):
            res = {}
            for image in os.listdir(spath):
                ipath = spath + "/" + image
                iname = image.split(".")[0].capitalize()
                res[iname] = imConv(ipath)
            json2file(res, f"json/{folder}.json")
        else:
            mainImages.append(folder)    
    res = {}
    for image in mainImages:
        ipath = path + "/" + image
        iname = image.split(".")[0].capitalize()
        res[iname] = imConv(ipath)
    json2file(res, "main.json")
            

if __name__ == "__main__":
    if (len(sys.argv) < 2):
        print("No nonogram directory specified")
    elif not os.path.exists(sys.argv[1]):
        print("Specified nonogram directory does not exist")
    else:
        if not os.path.exists("json"):
            os.makedirs("json")
        loadImages(path=sys.argv[1])
